/*
 * Femto, a fictive computer emulator
 * Copyright (C) 2021 Semperfis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* Computer architecture:
 * - 4KBs RAM
 * - RISC CPU: 4 GP REGISTERS; INTEGER ONLY; REDUCE ADDRESSING MODES & MEMORY
 * - STRUCTURE OF FLAGS REGISTER: XXXX INCZ (I : INTERRUPT; N : Negative; C : Carry; Z : Zero)
 * - INSTRUCTION FORMAT: (I: INST; M : ADDRESSING MODES; R : REGISTERS; D : DATA; A : ADDRESS)
 * - MIII IIII   RRRR xxxx   DDDD DDDD
 * - MIII IIII   RRRR AAAA   AAAA AAAA
 */

#ifndef CPU_H_
#define CPU_H_

#include "../femto.h"


#define DREG  emu->dreg
#define SREG  emu->sreg
#define R     emu->r
#define SP    emu->sp
#define PC    emu->pc
#define RAM   emu->ram
#define F     emu->f
#define FLAGS emu->flags
#define INST  emu->inst
#define ADRM  emu->adrm
#define DATA  emu->data
#define HALT  emu->halt
#define ADDR  emu->addr
#define TEMP  emu->temp

void CpuExecInst(FemtoEmu_t *emu, bool verbose);
void StackPushByte(FemtoEmu_t *emu, uint8_t byte);
uint8_t StackPopByte(FemtoEmu_t *emu);

#endif